<?php
// Copyright (C) 2014 TeemIp
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2014 TeemIp
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

class _IPAddress extends IPObject
{
	/**
	 * Get the subnet mask of the subnet that the IP belongs to, if any.
	 */
	function GetSubnetMaskFromIp()
	{
		return "";
	}
	
	/**
	 * Get the gateway of the subnet that the IP belongs to, if any.
	 */
	function GetSubnetGatewayFromIp()
	{
		return "";
	}
	/**
	 * Check if IP's FQDN is unique.
	 */
	function IsFqdnUnique()
	{
		$sOrgId = $this->Get('org_id');
		if ($this->IsNew())
		{
			$iKey = -1;
		}
		else
		{
			$iKey = $this->GetKey();
		}
		$sFqdn = $this->Get('fqdn');
		
		// The check takes into account the global parameters that defines if duplicate FQDNs are authorized or not
		$sIpAllowDuplicateName = utils::ReadPostedParam('attr_ip_allow_duplicate_name', '');
		if (empty($sIpAllowDuplicateName))
		{
			$sIpAllowDuplicateName = IPConfig::GetFromGlobalIPConfig('ip_allow_duplicate_name', $sOrgId);
		}
		if ($sIpAllowDuplicateName == 'ipdup_no')
		{
			if ($sFqdn != "")
			{
				$oIpSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPAddress AS i WHERE i.fqdn = '$sFqdn' AND i.org_id = $sOrgId AND i.id != $iKey"));
				// Match for creations is verbiden. Match for modifications as well unless the current name is kept
				while ($oIp = $oIpSet->Fetch())
				{
					// Check status of IP before complaining - Released IPs don't count.
					if ($oIp->Get('status') != 'released')
					{
						return false;
					}
				}
			}
		}
		return true;
	}

	/*
	 * Compute attributes before writing object 
	 */     
	public function ComputeValues()
	{
		// Set FQDN
		$sShortName = $this->Get('short_name');
		if ($sShortName != "")
		{
			$sDomainName = $this->Get('domain_name');
			if ($sDomainName != "")
			{
				// Domain names are always fully qualified
				$this->Set('fqdn', $sShortName.'.'.$sDomainName);			
			}
			else
			{
				$this->Set('fqdn', $sShortName.'.');			
			}
		}
		else
		{
			$this->Set('fqdn', '');			
		}
	}

	/*
	 * Computes and display specific tabs
	 */
	function DisplayBareRelations(WebPage $oPage, $bEditMode = false)
	{
		// Execute parent function first 
		parent::DisplayBareRelations($oPage, $bEditMode);
			
		$sOrgId = $this->Get('org_id');
		if ($sOrgId != null)
		{
			if ($bEditMode)
			{
				// Tab for Global Parameters
				$oPage->SetCurrentTab(Dict::Format('Class:IPAddress/Tab:globalparam'));
				$oPage->p(Dict::Format('UI:IPManagement:Action:Modify:GlobalConfig'));
				$oPage->add('<table style="vertical-align:top"><tr>');
				$oPage->add('<td style="vertical-align:top">');
				
				if ($this->IsNew())
				{
					$sParameter = array ('ip_allow_duplicate_name', 'ping_before_assign', null);
				}
				else
				{
					$sParameter = array ('ip_allow_duplicate_name', null);
				}
				$this->DisplayGlobalParametersInLocalModifyForm($oPage, $sParameter);
				
				$oPage->add('</td>');
				$oPage->add('</tr></table>');
			}
			else
			{
				$iKey = $this->GetKey();

				// Tab for NAT IPs
				$oNatIpSearch = DBObjectSearch::FromOQL("SELECT lnkIPAdressToIPAddress AS ln WHERE (ln.ip1_id = $iKey OR ln.ip2_id = $iKey)");
				$oNatIpSet = new CMDBObjectSet($oNatIpSearch);
				$oIpSet = array();
				$iCountIp1 = 0;
				$iCountIp2 = 0;
				while ($oNatIp = $oNatIpSet->fetch())
				{
					if ($oNatIp->Get('ip1_id') == $iKey)
					{
						$iIpKey = $oNatIp->Get('ip2_id');
						$iCountIp1++;
					}
					else
					{
						$iIpKey = $oNatIp->Get('ip1_id');
						$iCountIp2++;
					}
					$oIpSet[] = MetaModel::GetObject('IPAddress', $iIpKey, false);
				}
				$oSet = CMDBObjectSet::FromArray('IPAddress', $oIpSet);
				// Ugly: this is to handle the current (2.0.3) iTop's limitation that doesn't handle the symetrical AttributeLinkedSetIndirect to lnkIPAdressToIPAddress
				// Note that $oPage->FinTab doesn't work...
				$oPage->RemoveTab(Dict::Format('Class:IPAddress/Attribute:ip_list'));
				$oPage->RemoveTab(Dict::Format('Class:IPAddress/Tab:ip_list', $iCountIp1));
				$oPage->SetCurrentTab(Dict::Format('Class:IPAddress/Tab:ip_list', $oNatIpSet->Count()));
				$oPage->p(MetaModel::GetClassIcon('IPAddress').'&nbsp;'.Dict::Format('Class:IPAddress/Tab:ip_list+'));
				if ($oNatIpSet->Count() != 0)
				{
					$oBlock = DisplayBlock::FromObjectSet($oSet, 'list');
					$oBlock->Display($oPage, 'nat_ips', array('menu' => false));
				}
				
				// Tab for CIs using the IP
				$aCIsToList = array(
					'DatacenterDevice' 	=> array('ip_attribute' => 'managementip_id'),
					'VirtualMachine' 	=> array('ip_attribute' => 'managementip_id'),
					'MobilePhone' 		=> array('ip_attribute' => 'ipaddress_id'),
					'IPPhone' 			=> array('ip_attribute' => 'ipaddress_id'),
					'Tablet' 			=> array('ip_attribute' => 'ipaddress_id'),
					'Peripheral'		=> array('ip_attribute' => 'ipaddress_id'),
					'PC' 				=> array('ip_attribute' => 'ipaddress_id'),
					'Printer' 			=> array('ip_attribute' =>'ipaddress_id'),
				);
				
				$iNbAllCIs = 0;
				foreach ($aCIsToList as $sCI => $sKey)
				{
					if  (MetaModel::IsValidClass($sCI))
					{
						$sIPAttribute = $aCIsToList[$sCI]['ip_attribute'];
						$oCISearch = DBObjectSearch::FromOQL("SELECT $sCI WHERE $sIPAttribute = $iKey");
						$oCISet = new CMDBObjectSet($oCISearch);
						if (method_exists($oCISet,'SetShowObsoleteData'))
						{
							// For compatibility with iTop versions older than 2.4
							$oCISet->SetShowObsoleteData(utils::ShowObsoleteData());
						}
						$iNbCIs = $oCISet->Count();
						$aCIsToList[$sCI]['nb_to_list'] = $iNbCIs;
						$aCIsToList[$sCI]['set'] = $oCISet;
						$iNbAllCIs += $iNbCIs;
					}
				}

				$oIPInterfaceToIPAddressSearch = DBObjectSearch::FromOQL("SELECT lnkIPInterfaceToIPAddress AS l WHERE l.ipaddress_id = $iKey");
				$oIPInterfaceToIPAddressSet = new CMDBObjectSet($oIPInterfaceToIPAddressSearch);
				if (method_exists($oIPInterfaceToIPAddressSet,'SetShowObsoleteData'))
				{
					// For compatibility with iTop versions older than 2.4
					$oIPInterfaceToIPAddressSet->SetShowObsoleteData(utils::ShowObsoleteData());
				}
				$iNbIPInterfaces = $oIPInterfaceToIPAddressSet->Count();
				$iNbAllCIs += $iNbIPInterfaces;
				$oIPInterfaceSet = array();
				while ($oLnk = $oIPInterfaceToIPAddressSet->fetch())
				{
					$iIpIntKey = $oLnk->Get('ipinterface_id');
					$oIPInterfaceSet[] = MetaModel::GetObject('IPInterface', $iIpIntKey, false);
				}
				$oSet = CMDBObjectSet::FromArray('IPInterface', $oIPInterfaceSet);
				
				$oPage->SetCurrentTab(Dict::Format('Class:IPAddress/Tab:ci_list', $iNbAllCIs));
				if ($iNbAllCIs != 0)
				{
					$oPage->p(MetaModel::GetClassIcon('FunctionalCI').'&nbsp;'.Dict::Format('Class:IPAddress/Tab:ci_list+', $iNbCIs));
					foreach ($aCIsToList as $sCI => $sKey)
					{
						if  ($aCIsToList[$sCI]['nb_to_list'] != 0)
						{
							$oBlock = DisplayBlock::FromObjectSet($aCIsToList[$sCI]['set'], 'list');
							$oBlock->Display($oPage, $sCI, array('menu' => false));
						}
					}
					
					if ($iNbIPInterfaces != 0)
					{
						$oBlock = DisplayBlock::FromObjectSet($oSet, 'list');
						$oBlock->Display($oPage, 'ii_id', array('menu' => false));
					}
				}
				else
				{
					$oPage->p(MetaModel::GetClassIcon('FunctionalCI').'&nbsp;'.Dict::Format('Class:IPAddress/Tab:NoCi+'));
				}

				
				// Tab for related IP Requests, if any
				if  (MetaModel::IsValidClass('IPRequestAddress'))
				{
					$oIpRequestSearch = DBObjectSearch::FromOQL("SELECT IPRequestAddress AS r WHERE r.ip_id = $iKey");
					$oIpRequestSet = new CMDBObjectSet($oIpRequestSearch);
					if (method_exists($oIpRequestSet,'SetShowObsoleteData'))
					{						
						// For compatibility with iTop versions older than 2.4
						$oIpRequestSet->SetShowObsoleteData(utils::ShowObsoleteData());
					}
					$sCount = $oIpRequestSet->Count();
					$oPage->SetCurrentTab(Dict::Format('Class:IPAddress/Tab:requests', $sCount));
					$oPage->p(MetaModel::GetClassIcon('IPRequestAddress').'&nbsp;'.Dict::Format('Class:IPAddress/Tab:requests+'));
					$oBlock = new DisplayBlock($oIpRequestSearch, 'list');
					$oBlock->Display($oPage, 'ip_requests', array('menu' => false));
				}
			}
		}
  	}
  	
  	/**
	 * Check validity of new IP attributes before creation
	 */
	public function DoCheckToWrite()
	{
		// Run standard iTop checks first
		parent::DoCheckToWrite();
		
		// Make sure name doesn't already exist (matches)
		if (! $this->IsFqdnUnique())
		{
			return (Dict::Format('UI:IPManagement:Action:New:IPAddress:IPNameCollision'));
		}
	}
	
	/**
	 * Change flag of attributes that shouldn't be modified beside creation.
	 */
	public function GetAttributeFlags($sAttCode, &$aReasons = array(), $sTargetState = '')
	{
		switch ($sAttCode)
		{
			case 'fqdn':
			case 'responds_to_ping':
			case 'responds_to_iplookup':
			case 'fqdn_from_iplookup':
			case 'responds_to_scan':
				return OPT_ATT_READONLY;
				
			default:
				break;
		}
		if ((!$this->IsNew()) && ($sAttCode == 'org_id'))
		{
			return OPT_ATT_READONLY;
		}
		return parent::GetAttributeFlags($sAttCode, $aReasons, $sTargetState);
	}
	
	/**
	 * Change flag of attributes that shouldn't be modified at creation.
	 */
	public function GetInitialStateAttributeFlags($sAttCode, &$aReasons = array())
	{
		switch ($sAttCode)
		{
			case 'fqdn':
			case 'responds_to_ping':
			case 'responds_to_iplookup':
			case 'fqdn_from_iplookup':
			case 'responds_to_scan':
				return OPT_ATT_READONLY;
				
			default:
				break;
		}
		return parent::GetInitialStateAttributeFlags($sAttCode, $aReasons);
	}
	
	/**
	 * Manage status of IP when attached to a device 
	 */					   
	public static function SetStatusOnAttachment ($iIpId = null, $iPreviousIpId = null)
	{
		if ($iIpId != $iPreviousIpId) 
		{
			if ($iIpId != null)
			{
				$oIP = MetaModel::GetObject('IPAddress', $iIpId, false /* MustBeFound */);
				if ($oIP != null)
				{
					if ($oIP->Get('status') != 'allocated')
					{
						$oIP->Set('status', 'allocated');	
						$oIP->DBUpdate();
					}
				}
			}
			if ($iPreviousIpId != null)
			{
				$oIP = MetaModel::GetObject('IPAddress', $iPreviousIpId, false /* MustBeFound */);
				if ($oIP != null)
				{
					if ($oIP->Get('status') == 'allocated')
					{
						$oIP->Set('status', 'unassigned');	
						$oIP->DBUpdate();
					}
				}
			}
		}
	}
	
	/**
	 * Manage status of IP when deattached from a device 
	 */
	public static function SetStatusOnDetachment ($iIpId = null)
	{
		if ($iIpId != null)
		{
			$oIP = MetaModel::GetObject('IPAddress', $iIpId, false /* MustBeFound */);
			if ($oIP != null)
			{
				if ($oIP->Get('status') == 'allocated')
				{
					$oIP->Set('status', 'unassigned');
					$oIP->DBUpdate();
				}
			}
		}
	}
	
}
